/*                Least Square Parameter Estimation Program                    */
/*                   for Differencial Equation Ver. 3.0                        */
/*                       Written by Yasuhisa Kuroda                            */
/*                 Modified for DJGPP in 2002 by Y. Kuroda                     */
/*          Modified for Visual Studio C++ in 2014 by Y. Kuroda                */
/*  This program numerically solves a set of differential equations by         */
/*  Runge-Kutta-Gill method and executes the least square parameter estimation */
/*  using user's data set n*(t,x[i]) (n<=1000).                                */
/*  Equations, dx[i]/dt=f(x[i],t,p[j],c[k]),                                   */
/*  where  x[i] and t are dependent and indipendent variables,                 */
/*         p[j] are parameters and c[k] are condition parameter,               */
/*  are defined as dx[i]=f(x[i],t,p[j],c[k]) (i,j,k<=10) in function().        */
/*  If initial x[i]'s must be calculated, define necessary equations to        */
/*  determined these values in function2() (Initial x[i]=g(x[j],p[i],c[i])).   */

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <setjmp.h>
#include <ctype.h>
#include <string.h>
#define MAX_PARA 21
#define MAX_x 21
#define MAX_c 11
#define MAX_condition 11

char *me[4],*outdv,sbnm[50];
long nn[11];
int m,n,nc,ncp,nt,tn,lsm,fnt,wg1,wg2,qyn,pnt[MAX_condition],nx,obx[MAX_x];
double sq1,sq2,dx[MAX_x],x[MAX_x],p[MAX_PARA],c[MAX_c],zc[MAX_condition][MAX_c];
double t0[MAX_condition],tf[MAX_condition],x0[MAX_condition][MAX_x];
double d,t,x1[MAX_condition][MAX_x],q[MAX_x],e[MAX_x],rr[MAX_x],dt[MAX_condition];
jmp_buf jumper;

double sum(double *data[],double *a[],double *xp[],double *wg[]);
void jacobian(long *tp,double *xp[],double *xpp[],double p1[],double *a[],double *b[],double *wg[]);
int gram_schmidt(double *a[],double del[],double r[][MAX_PARA],int iq[]);
void rkg(double *xp[],long *tp);
double trace(double r[][MAX_PARA]);
void grdata(double p2[],double *data[]);
void error(int rank,int iq[],double r[][MAX_PARA]);
void finalout(int cy,int rank,int iq[],double ss2,double p[],double r[][MAX_PARA],double *data[],double *xp[],int org);
void function(void);
void cycleprint(double p1[],double p2[],double ss,int cy,int mess);
void paraset();
void function2();

main ()
 {
  FILE *inpt;
  int i,j,k,l,i1,j1,nnn,fl1,rank,iq[MAX_PARA],cy,mess,maxl,errval;
  long *tp,mnn;
  double p1[MAX_PARA],p2[MAX_PARA],r[MAX_PARA][MAX_PARA],sl,sl1,rr1,tra,u;
  double del[MAX_PARA],ss1,ss2,damp,lamda,slamda,cc1,cc2;
  double *a[MAX_PARA],*b[MAX_PARA],*xp[MAX_x];
  double *data[MAX_x],*xpp[MAX_x],*wg[MAX_x];
  char an[1],*percent;

  me[0]="Simulation";
  me[1]="Damping Gauss-Newton";
  me[2]="Marquardt";
  me[3]="Failed in Memory Acquisition.\n";
  sq1=1-sqrt(0.5);sq2=1+sqrt(0.5);
  maxl=100;
  percent="%";
  
  printf("REACTION RATE ANALYSIS PROGRAM v.2.5 \n");
  printf("   (Originaly Written by Y. Kuroda for DOS,\n");
  printf("    Modified for DJGPP by Y. Kuroda in 2002,\n");
  printf("    Kyoto Institute of Technology.)\n\n");
  printf("Job Type :\n");
  for(i=0;i<=2;i++)
    printf(" %d) %s\n",i,me[i]);
  printf("Which method do you select ?  ");
  scanf("%d",&lsm);  printf("\n");

/* simulation only  */  
  if (lsm==0)
   {printf("Do you have observed data ? (Y/N)  ");
/*   fflush(stdin);qyn=getchar();                  */
     scanf("%s",&qyn);
     if ((qyn=='N')||(qyn=='n'))
      {printf("\nSubject Name        : ");scanf("%s",sbnm);
       printf("Number of Equation (dx[i]/dt) : ");scanf("%d",&m);
       printf("Number of Parameter p[i]      : ");scanf("%d",&n);
       printf("How many different condition ?  ");scanf("%d",&nc);
       printf("How many condition parameters c[i] ?  ");scanf("%d",&ncp);
       for(i1=1;i1<=nc;i1++)
        {printf("Condition(%d)\n",i1);
		 for(j1=1;j1<=ncp;j1++)
		  {printf(" c[%d] = ",j1);scanf("%lf",&zc[i1][j1]);}}
	   goto out3;}}
/* end of setting of simulation conditions (goto out3) */

  qyn='Y';
  printf("Number of Equation (dx[i]/dt) : ");scanf("%d",&m);
  printf("Number of Parameter p[i]      : ");scanf("%d",&n);
  printf("Weight for Data :\n");
  printf(" 1) None  2) 1/y[i]  3) User's Weight  ");scanf("%d",&wg1);
   
/*  file read  */
  printf("Data File Name  :   ");scanf("%s",sbnm);
  inpt=fopen(sbnm,"r");
  fscanf(inpt,"%d %d %d ",&nx,&nc,&ncp);
/*  printf("%d , %d , %d\n",nx,nc,ncp);     */
  for (i=1;i<=nx;i++)
    fscanf(inpt,"%d ",&obx[i]);
  fscanf(inpt,"%d ",&nt);
  for (i=0;i<=nx;i++)
    {
     if ((data[i]=calloc(nt+1,sizeof(double)))==NULL)
      {printf("%s",me[3]);gets(an);exit(0);}
     if ((wg[i]=calloc(nt+1,sizeof(double)))==NULL)
      {printf("%s",me[3]);gets(an);exit(0);}
    }
  for (nnn=1,i=1;i<=nc;i++)
    {
     for (j=1;j<=ncp;j++)
       fscanf(inpt,"%lf ",&zc[i][j]);
     fscanf(inpt,"%d ",&pnt[i]);
	 for (j=1;j<=pnt[i];j++)
	  {
	   fscanf(inpt,"%lf ",&data[0][nnn]);
	   for (k=1;k<=nx;k++)
		{fscanf(inpt,"%lf ",&data[k][nnn]);
		 if (wg1==3) fscanf(inpt,"%lf ",&wg[k][nnn]);}
	   nnn++;
      }
	 }
   fclose(inpt);

   printf("Number of Condition : %d\n",nc);
   printf("Number of Condition Parameter : %d\n",ncp);
   if (ncp!=0)
     {
      for(i1=1;i1<=nc;i1++)
        {printf("Condition(%d)\n",i1);
		 for(j1=1;j1<=ncp;j1++)
		  {printf(" c[%d] = %lf, ",j1,zc[i1][j1]);}
		 printf("\n");}
     }
   printf("x[i]'s observed are ");for (i=1;i<=nx;printf("%d,",obx[i]),i++);printf("\n");
   for (i=1;i<=nc;i++)
     printf("Data Points under Condition (%d) : %d\n",i,pnt[i]);
   tn=nt*nx;
   if (lsm==2)
      {for (i=0;i<=n;b[i]=calloc(tn+1,sizeof(double)),i++);wg2=tn+n+1;}
   else wg2=tn+1;
   for (i=0;i<=n;i++)
      {if ((a[i]=calloc(wg2,sizeof(double)))==NULL)
	    {printf("%s",me[3]);gets(an);exit(0);}}
   for (i=1;i<=nx;i++)
      {if ((xpp[i]=calloc(nt+1,sizeof(double)))==NULL)
	    {printf("%s",me[3]);gets(an);exit(0);}}
/*  end of file read  */

/* data weighting  */
  switch(wg1)
    {
     case 1:for(i=1;i<=nx;i++)
	      {for(j=1;j<=nt;j++)
	         wg[i][j]=1.0;}
	      break;
     case 2:for(lamda=1.0,i=1;i<=nx;i++)
	         {for(j=1;j<=nt;j++)
		       {slamda=fabs(data[i][j]);
		        if ((lamda>=slamda)&&(data[i][j]!=0)) lamda=slamda;}}
	        for(i=1;i<=nx;i++)
	         {for(j=1;j<=nt;j++)
		       {if (data[i][j]==0) wg[i][j]=1.0/lamda;
		        else wg[i][j]=fabs(1.0/data[i][j]);}}
	        break;
     case 3:for(i=1;i<=nx;i++)
	         {for(j=1;j<=nt;j++)
		       wg[i][j]=fabs(1.0/wg[i][j]);}
    }
  
out3:

  if ((tp=calloc(nt+2,sizeof(long)))==NULL)
   {printf("%s",me[3]);exit(0);}
  for (i=1;i<=m;i++)
   {if ((xp[i]=calloc(nt+2,sizeof(double)))==NULL)
     {printf("%s",me[3]);exit(0);}}
     
/* initialization  */
start:
  mnn=0;
  for(l=1;l<=nc;l++)
   {
    printf("For condition[%d]\n",l);
	printf("  Initial t ? ");scanf("%lf",&t0[l]);
    printf("  Final t   ? ");scanf("%lf",&tf[l]);
	printf("  Computing interval ? ");scanf("%lf",&dt[l]);
    nn[l]=(long)((tf[l]-t0[l])/dt[l]);mnn+=nn[l];
   }
  printf("\n");
startI:
  printf("If inititial x[i]'s must be calculated, set 99999 as the x[i]'s.\n");
  fnt=0;
  for(i1=1;i1<=nc;i1++)
   {
    printf(" For condition [%d]\n",i1);    
    for(i=1;i<=m;i++)
     {
      printf("  Initial x[%d] = ",i);
	  scanf("%lf",&x1[i1][i]);x0[i1][i]=x1[i1][i];
	  if (x0[i1][i]==99999) fnt=1;
     }
   }
  printf("\n");
startII:
  if (lsm==0) {grdata(p2,data);goto out2;}
  printf("Convergence Condition :\n");
  printf("  1) Standard (delta s.s < 0.001%s , p[i] < 0.01%s) , 2) User's set.  ",percent,percent);
  scanf("%d",&i);
  if (i==1)
   {cc1=0.00001;cc2=0.0001;}
  else
   {
    printf("Delta ss (%s) = ",percent);scanf("%lf",&cc1);cc1/=100;
    printf("Delta p[i] (%s) = ",percent);scanf("%lf",&cc2);cc2/=100;
   }
  if (lsm==1)
    {
     printf("Set damping factor (Standerd value = 1)  ");
     scanf("%lf",&damp);
    }
    
/* start clculation  */
/*  printf("\nStart computation.\nIf you want to break computation, hit 'ctrl'+'c' key.\n\n");   */
  printf("Maximum Loop (Standard = 100) : ");scanf("%d",&maxl);printf("\n");
  for(i=1;i<=n;i++)
   {printf("Initial p[%d] = ",i);scanf("%lf",&p1[i]);p2[i]=p1[i];}

  errval=setjmp(jumper);
  if (errval!=0)
   {
	printf("\nError. Start again with different conditions.\n");
	goto out2;
   }

  if (lsm==0) {grdata(p2,data);goto out2;}
  d=0.000001;j=1;nn[0]=0;
  for(i1=1;i1<=nc;i1++)
   {
    for(i=1;i<=pnt[i1];i++)
     {tp[j]=(long)(data[0][j]/dt[i1])+nn[0];j++;}
    nn[0]+=nn[i1];
   }
  for(i=1;i<=n;i++)
    p[i]=p2[i];
  rkg(xp,tp);
  ss2=sum(data,a,xp,wg);
  printf("Initial ss = %-g\n\n",ss2);
  cy=mess=1;fl1=0;
  for(i=1;i<=n;i++)
    p1[i]=p[i];
  if (lsm==2) {lamda=0;goto marquardt;}
  do
   {
    ss1=ss2;
    do
     {
	  jacobian(tp,xp,xpp,p1,a,b,wg);rank=gram_schmidt(a,del,r,iq);
	  if (fl1>0) {printf("\n(Loop %d)\nFail to find smaller ss.\n\n",cy);}
	  do
	   {
	    for (i=1;i<=rank;i++)
		  r[i][0]=del[i]*damp;
	    for (i=1;i<=rank;i++)
	      p2[iq[i]]=p1[iq[i]]+r[iq[i]][0];
	    for(i=1;i<=n;i++)
	      p[i]=p2[i];
	    rkg(xp,tp);
	    ss2=sum(data,a,xp,wg);
        if (ss1>ss2) goto out1;
	    damp*=0.5;
       }
      while (damp>0.00005);
	  fl1++;
	  if (fl1>5)
	   {for (i=1;i<=n;i++)
	      p2[i]=p1[i];
	    printf("\nPresent computation gives no smaller ss.\n");
	    printf("Stop under Non-convergent Conditions.\nPresent Status ;\n");
	    goto fprn;}
	  for (i=1;i<=n;i++)
	    p1[i]=p2[i];
      damp*=2;
     }
    while (ss2>1.00001*ss1);
    mess=2;
out1:
    if (damp<1) damp*=2;
    cycleprint(p1,p2,ss2,cy,mess);mess=1;
    cy++;fl1=0;
    if (cy>maxl) 
       {printf("\nPresent computation exceeds loop maximum.\n");
        printf("Stop in Non-convergent Condition.\nPresent Status ;\n");goto fprn;}
    for (j=1,i=1;i<=n;i++)
      j*=(fabs(r[i][0])<fabs(p2[i])*cc2);
   }
 while (((ss1-ss2)>ss2*cc1)||(j!=1));
 jacobian(tp,xp,xpp,p2,a,b,wg);
 rank=gram_schmidt(a,del,r,iq);goto fprn;

marquardt:
 do
  {
   ss1=ss2;jacobian(tp,xp,xpp,p1,a,b,wg);
   for (i=1;i<=tn;b[0][i]=a[0][i],i++);
   for (k=1;;)
    {
     for (i=1;i<=n;i++)
	  {for (j=0;j<=n;j++)
	    a[j][tn+i]=0;}
     slamda=sqrt(lamda);
     for (i=1;i<=n;i++)
       a[i][tn+i]=slamda;
	 tn+=n;
     rank=gram_schmidt(a,del,r,iq);
     tn-=n;
	 for (i=1;i<=rank;i++)
	   p2[iq[i]]=p1[iq[i]]+del[iq[i]];
	 for(i=1;i<=n;i++)
	   p[i]=p2[i];
	 rkg(xp,tp);
	 ss2=sum(data,a,xp,wg);sl=0;
	 for (i=1;i<=rank;i++)
	  {
	   sl1=0;
	   for (j=1;j<=rank;j++)
	     sl1+=del[iq[j]]*r[iq[i]][iq[j]];
	   sl=sl+sl1*sl1+lamda*del[iq[i]]*del[iq[i]];
	  }
	 sl=-sl;rr1=(ss2-ss1)/sl;
	 if (rr1>0.75)
	  {
	   lamda/=2;error(rank,iq,r);tra=trace(r);
	   if (lamda<tra) lamda=0;
	  }
	 else
	  {
	   if (rr1<0.25)
	     {
	      u=2-rr1;
	      if (u>10) u=10;else if (u<2) u=2;
	      if (lamda<=0)
		    {error(rank,iq,r);tra=trace(r);lamda=tra;u/=2;}
          lamda*=u;
	      if (lamda>1e6)
		    {
		     printf("\nLamda exceeds limit value (1000000).\n");
		     printf("Present computation gives no smaller ss.\n");
	         printf("Stop under Non-convergent Conditions.\nPresent Status ;\n");
	         goto fprn;
		    }
       }
	  }
   if (ss1>ss2)
	 {
	  if (lamda>10*tra) mess=3;
	  break;
	 }
   else
    {
	 if (ss2<1.00001*ss1)
	  {
	   mess=2;
	   break;
      }
    }
   for (i=1;i<=n;i++)
     p2[i]=p1[i];
   for (i=0;i<=n;i++)
	 {for (j=1;j<=tn;j++)
	   a[i][j]=b[i][j];}
   }
  cycleprint(p1,p2,ss2,cy,mess);mess=1;
  cy++;
  if (cy>maxl) 
       {printf("\nPresent computation exceeds loop maximum.\n");
        printf("Stop in Non-convergent Condition.\nPresent Status ;\n");goto fprn;}
  for (j=1,i=1;i<=n;i++)
    j*=(fabs(del[i])<fabs(p2[i])*cc2);
  }
 while (((ss1-ss2)>ss2*cc1)||(j!=1));
 jacobian(tp,xp,xpp,p2,a,b,wg);rank=gram_schmidt(a,del,r,iq);
 
fprn:
  error(rank,iq,r);
  finalout(cy,rank,iq,ss2,p2,r,data,xp,1);
out2:
  for (i=1;;)
    {
     printf("\nAnother method(0,1,2) or Output results(3) or Graphic data(4) or End(5)  ");
     scanf("%d",&j);printf("\n");
     switch (j)
       {
	    case 0:if (lsm==2) 
	            for(i=0;i<=n;i++)
	              free(b[i]);
		       lsm=0;
		       for(i1=1;i1<=nc;i1++)
			    {for(j1=1;j1<=ncp;j1++)
			       {printf(" c[%d] = ",j1),scanf("%lf",&zc[i1][j1]);}}
	           goto start;
	    case 1:if (lsm==2) 
	             {for(i=0;i<=n;i++)
	                free(b[i]);}
			   lsm=1;
			   printf("\nChange from Calc.Domain(1) or Initial x(2) or Initial Parameter(3)  ");
			   scanf("%d",&k);printf("\n");
			   switch (k)
				 {
				  case 1:goto start;
				  case 2:goto startI;
				  case 3:goto startII;
				 }
		case 2:if (lsm!=2)
				{for(i=0;i<=n;i++)
			       free(a[i]);
			     for(i=0;i<=n;i++)
			       b[i]=calloc(tn+1,sizeof(double));
			     for(i=0;i<=n;i++)
			       a[i]=calloc(tn+n+1,sizeof(double));}
			   lsm=2;
			   printf("\nChange from Calc.Domain(1) or Initial x(2) or Initial Parameter(3)  ");
			   scanf("%d",&k);printf("\n");
			   switch (k)
				 {
				  case 1:goto start;
				  case 2:goto startI;
				  case 3:goto startII;
				 }
	    case 3:finalout(cy,rank,iq,ss2,p2,r,data,xp,2);
		       break;
	    case 4:grdata(p2,data);break;
	    case 5:goto end;
       }
    }
end:;
/*  return 0;  */
/*  ExitToShell();    */
  }    /*   END OF MAIN PROGRAM    */

double sum(double *data[],double *a[],double *xp[],double *wg[])
 {
  int i,j,jj=0;
  double ss=0;
  for (i=1;i<=nx;i++)
    {
     for (j=1;j<=nt;j++)
       {
	    a[0][jj+j]=(data[i][j]-xp[obx[i]][j])*wg[i][j];
	    ss+=a[0][jj+j]*a[0][jj+j];
       }
     jj+=nt;
    }
  return(ss);
 }    /*  END OF SUM  */

void jacobian(long *tp,double *xp[],double *xpp[],double p1[],double *a[],double *b[],double *wg[])
 {
  int i1,j1,k,kk;
  double dp;
  for (i1=1;i1<=nx;i1++)
    {for (j1=1;j1<=nt;xpp[i1][j1]=xp[obx[i1]][j1],j1++);}
  for (i1=1;i1<=n;i1++)
    {
     dp=p1[i1]*d;
     p[i1]=p1[i1]-dp;
     rkg(xp,tp);kk=0;
     for (j1=1;j1<=nx;j1++)
	   {for (k=1;k<=nt;k++)
		  {a[i1][k+kk]=(xpp[j1][k]-xp[obx[j1]][k])*wg[j1][k]/dp;
		   if (lsm==2) b[i1][k+kk]=a[i1][k+kk];}
		kk+=nt;}
     p[i1]=p1[i1];
    }
 }    /*  END OF JACOBIAN  */

void rkg(double *xp[],long *tp)
 {
  int i,j,k,l,ckc;
  long ii,jj;
  for (ii=0,ckc=1,i=1;i<=nc;i++)
    {
     for (t=t0[i],jj=0,j=1;j<=ncp;j++)
       c[j]=zc[i][j];
	 for (k=1;k<=m;k++)
	   {q[k]=0;x[k]=x0[i][k];}
     if (fnt==1) function2();
     for (k=1;k<=nn[i];k++)
      {
	   if (ii==tp[ckc]) 
	     {for(l=1;l<=m;l++)
	        xp[l][ckc]=x[l];
	      ckc+=1;}
	   ii+=1;
	   jj+=1;
	   function();
	   for (l=1;l<=m;l++)
	     {e[l]=dt[i]*dx[l];rr[l]=(e[l]-2*q[l])/2;x[l]+=rr[l];q[l]+=(3*rr[l]-e[l]/2);}
	   function();
	   for (l=1;l<=m;l++)
	     {e[l]=dt[i]*dx[l];rr[l]=(e[l]-q[l])*sq1;x[l]+=rr[l];q[l]+=(3*rr[l]-e[l]*sq1);}
	   function();
	   for (l=1;l<=m;l++)
	     {e[l]=dt[i]*dx[l];rr[l]=(e[l]-q[l])*sq2;x[l]+=rr[l];q[l]+=(3*rr[l]-e[l]*sq2);}
	   function();
	   for (l=1;l<=m;l++)
	     {e[l]=dt[i]*dx[l];rr[l]=(e[l]-2*q[l])/6;x[l]+=rr[l];q[l]+=(3*rr[l]-e[l]/2);}
	   t=t0[i]+jj*dt[i];
      }
    }
 }

int gram_schmidt(double *a[],double del[],double r[][MAX_PARA],int iq[])
 {
  int i,j,k,l,rank,idumy;
  double nr[MAX_PARA],nr1[MAX_PARA],z[MAX_PARA],fdumy;
  for (i=1;i<=n;i++)
    {
     nr[i]=0;iq[i]=i;
     for (j=1;j<=tn;j++)
       nr[i]+=a[i][j]*a[i][j];
    }
  for (i=1;i<=n;i++)
    {
     if (nr[1]>nr[i]) continue;
     fdumy=nr[1];nr[1]=nr[i];nr[i]=fdumy;
     idumy=iq[1];iq[1]=iq[i];iq[i]=idumy;
    }
  for (i=1;i<=n;i++)
     for (j=1;j<=n;j++)
       r[i][j]=0;
  nr1[1]=nr[1];
  for (i=1;i<=n;i++)
    {
     if (nr1[i]/nr[i]<0.000005) goto gsout;
     r[iq[i]][iq[i]]=sqrt(nr1[i]);
     for (j=1;j<=tn;j++)
       a[iq[i]][j]/=r[iq[i]][iq[i]];
     if (i==n) continue;
     for (k=i+1;k<=n;k++)
       {
	    for (l=1;l<=tn;l++)
	      r[iq[i]][iq[k]]+=a[iq[i]][l]*a[iq[k]][l];
	    for (l=1;l<=tn;l++)
	      a[iq[k]][l]-=a[iq[i]][l]*r[iq[i]][iq[k]];
       }
     for (j=i+1;j<=n;j++)
	 for (nr1[j]=0,k=1;k<=tn;k++)
	   nr1[j]+=a[iq[j]][k]*a[iq[j]][k];
     if (i==n-1) continue;
     for (j=i+2;j<=n;j++)
       {
	    if (nr1[i+1]/nr[i+1]>nr1[j]/nr[j]) continue;
	    idumy=iq[i+1];iq[i+1]=iq[j];iq[j]=idumy;
	    nr1[i+1]=nr1[j];
	    fdumy=nr[i+1];nr[i+1]=nr[j];nr[j]=fdumy;
       }
    }
gsout:rank=i-1;
  for (i=1;i<=n;i++)
    del[i]=0;
  for (i=1;i<=rank;i++)
    {
     for (z[iq[i]]=0,j=1;j<=tn;j++)
       z[iq[i]]+=a[iq[i]][j]*a[0][j];
     for (k=1;k<=tn;k++)
       a[0][k]-=a[iq[i]][k]*z[iq[i]];
    }
  del[iq[rank]]=z[iq[rank]]/r[iq[rank]][iq[rank]];
  if (rank==1) goto gsend;
  for (i=rank-1;i>=1;i--)
    {
     for (del[iq[i]]=z[iq[i]],j=rank;j>=i+1;j--)
       del[iq[i]]-=r[iq[i]][iq[j]]*del[iq[j]];
     del[iq[i]]/=r[iq[i]][iq[i]];
    }
gsend:return(rank);
 }      /*   END OF GRAM-SCHUMIDT   */

void error(int rank,int iq[],double r[][MAX_PARA])
 {
  int i,j,k;
  for (i=rank;i>=1;i--)
    {
     r[iq[i]][iq[i]]=1/r[iq[i]][iq[i]];
     j=rank;
     while (((i+1)<=rank)&&(j>=i+1))
       {
	    r[iq[i]][iq[j]]*=r[iq[j]][iq[j]];
	    k=i+1;
	    while ((k<rank)&&(k<=j-1))
	     {
	      r[iq[i]][iq[j]]+=r[iq[i]][iq[k]]*r[iq[k]][iq[j]];
	      k++;
	     }
	    r[iq[i]][iq[j]]=-r[iq[i]][iq[j]]*r[iq[i]][iq[i]];
	    j--;
      }
    }
  for (i=1;i<=rank;i++)
     {
      r[iq[i]][iq[i]]*=r[iq[i]][iq[i]];
      for (j=i+1;j<=rank;j++)
        r[iq[i]][iq[i]]+=r[iq[i]][iq[j]]*r[iq[i]][iq[j]];
     }
 }   /*   END OF ERROR MATRIX   */

double trace(double r[][MAX_PARA])
 {
  int i;double tra;
  for (tra=0,i=1;i<=n;i++)
    tra+=r[i][i];
  tra=1/tra;
  return(tra);
 }   /*   END OF TRACE    */

void cycleprint(double p1[],double p2[],double ss,int cy,int mess)
 {
  int i;
  printf("Loop %d",cy);
  if (mess==2) printf("(Non smaller ss)");
  if (mess==3) printf("(Too large lamda)");
  printf("\n");
  for (i=1;i<=n;i++)
	{
	 printf("p[%d] = %-g\n",i,p2[i]);
	 p1[i]=p2[i];}
  printf("ss = %-g\n",ss);
 }      /*   END OF CYCLE-PRINT   */
 
void finalout(int cy,int rank,int iq[],double ss2,double p[],double r[][MAX_PARA],double *data[],double *xp[],int org)
 {
  FILE *out;
  char fl[50];
  int i,j,k,nnn;
  
  if (org==1)
     out=stdout;
  else
    {
     printf("Output on 1) Console or 2) Disk  ");scanf("%d",&org);
     if (org==1) 
       out=stdout;
     else   
       {printf("File name  :  ");scanf("%s",fl);
        out=fopen(fl,"w");
        printf("\n.......\n");}
    }
  
  fprintf(out,"\n* %s * by %s method\n",sbnm,me[lsm]);
  fprintf(out,"Iteration : %d\n",cy);
  fprintf(out,"      AIC : %-g\n",tn*log(ss2)+2*rank);
  if (rank==n)
    fprintf(out,"Full rank parameter.\n");
  else
    {
     fprintf(out,"Found rank deficiency.\n");
     for (i=1;i<=rank;i++)
       fprintf(out,"p[%d] ,",iq[i]);
     fprintf(out,"are normal.\n");
    }
  for (i=1;i<=n;i++)
    fprintf(out,"Final p[%d] = %-g  s.d. = %-g\n",i,p[i],sqrt(r[i][i]*ss2/(tn-n)));
  fprintf(out,"Final s.s. = %-g\n\n",ss2);
  
  if (org==1)
    {printf("1) Print Data Table     2) Next Step  ");scanf("%d",&i);}
  else
     i=1;
     
  if (i==1)  
    {fprintf(out,"\n   Calculated x[i] (Observed x[i])\n");
     fprintf(out," t");
     for (i=1;i<=nx;i++)
       fprintf(out,"    x[%d]        ",obx[i]);
     fprintf(out,"\n");
     for (nnn=1,i=1;i<=nc;i++)
      {
       for (j=1;j<=ncp;j++)
         fprintf(out,"c[%d] = %-g\n",j,zc[i][j]);
       for (j=1;j<=pnt[i];j++)
	    {fprintf(out,"%-g  ",data[0][nnn]);
	     for (k=1;k<=nx;k++)
	       fprintf(out,"%-g(%-g)  ",xp[obx[k]][nnn],data[k][nnn]);
	     fprintf(out,"\n");nnn++;}
        }
     fprintf(out,"\n");
    }
  if (org==2) fclose(out);
 }   /*   END OF PRINT OUT AS FILE  */


void grdata(double p2[],double *data[])
 {
  FILE *out;
  char fl[50],xxx[25];
  int i,ii,j,k,nox[11],ans,ans2,nnx,nnn,sim,z1z;
  double pint;
  long jj;
  
  ans2=3;
  printf("Graphic Data Output.\n\n");
  
  do
   {
    if (lsm==0) paraset();
    else
      {printf("Optimized results(1)  Calc.with given parameters(2)  ");scanf("%d",&sim);
       if (sim==2) paraset();
       else for(i=1;i<=n;i++) p[i]=p2[i];}
       
    askx:
      printf("Which x(i) do you select (1,2,3...) ?  ");scanf("%s",xxx);
      nnx=sscanf(xxx,"%d,%d,%d,%d,%d,%d,%d,%d,%d,%d",&nox[1],&nox[2],&nox[3],&nox[4],&nox[5],&nox[6],&nox[7],&nox[8],&nox[9],&nox[10]);
      for(i=1;i<=nnx;i++)
        {if ((nox[i]>m)||(nox[i]<=0)) {printf("Obs(%d) is Undefined Obs. !\n",i);goto askx;}}
    
	  printf("Condition No.  ");scanf("%d",&z1z);
	  printf("Output interval (Computing interval:%lf) : ",dt[z1z]);scanf("%lf",&pint);
	  sim=pint/dt[z1z];
	  if (sim==0) sim=1;
	if ((qyn=='Y')||(qyn=='y'))
	  {printf("\n1) With observed data 2) Calculated data only : ");
          scanf("%d",&ans);
         }

	if (ans2==3)
      {printf("File name  :  ");scanf("%s",fl);
       out=fopen(fl,"w");ans2=4;}
    else
      {printf("1) On file[%s]  2) Another file : ",fl);scanf("%d",&ans2);
       if (ans2==2) 
         {fclose(out);
          printf("File name  :  ");scanf("%s",fl);
          out=fopen(fl,"w");}
      }     
  
	printf("\n.......\n");

/*  Observed Data Output   */   
    if (ans==1)
	  {
          for (nnn=1,i=1;i<=nc;i++)
		{
	        for (j=1;j<=pnt[i];j++)
		  {
	          fprintf(out,"%-g ",data[0][nnn]);
	          for (k=1;k<=nx;k++)
	          fprintf(out,"%-g ",data[k][nnn]);
		   fprintf(out,"\n");
	         nnn++;
		  }
            fprintf(out,"\n");
		}
      } 

/*   Calcd. Data Output     */    
    for(t=t0[z1z],jj=0,i=1;i<=ncp;i++)
      c[i]=zc[z1z][i];
    for(i=1;i<=m;i++)
      {q[i]=0;x[i]=x0[z1z][i];}
	if (fnt==1) function2();
	ii=sim;
    for(j=1;j<=nn[z1z];j++)
	 {
	  if (ii==sim)
		{fprintf(out,"%-g ",t);
/*             fprintf(out,"\n%-g ",t);              */
		 for(k=1;k<=nnx;k++)
		 fprintf(out,"%-g ",x[nox[k]]);
               fprintf(out,"\n");              
		 ii=0;}
      jj+=1;
      function();
      for (i=1;i<=m;i++)
        {e[i]=dt[z1z]*dx[i];rr[i]=(e[i]-2*q[i])/2;x[i]+=rr[i];q[i]+=(3*rr[i]-e[i]/2);}
      function();
      for (i=1;i<=m;i++)
        {e[i]=dt[z1z]*dx[i];rr[i]=(e[i]-q[i])*sq1;x[i]+=rr[i];q[i]+=(3*rr[i]-e[i]*sq1);}
      function();
      for (i=1;i<=m;i++)
        {e[i]=dt[z1z]*dx[i];rr[i]=(e[i]-q[i])*sq2;x[i]+=rr[i];q[i]+=(3*rr[i]-e[i]*sq2);}
      function();
      for (i=1;i<=m;i++)
        {e[i]=dt[z1z]*dx[i];rr[i]=(e[i]-2*q[i])/6;x[i]+=rr[i];q[i]+=(3*rr[i]-e[i]/2);}
      t=t0[z1z]+jj*dt[z1z];
      ii++;
     }
    fprintf(out,"\n");
	printf("\n1) End of output  2) Other output : ");
       scanf("%d",&ans);
    }
  while(ans==2);

fclose(out);
endout:
  printf("\n\nEnd of File Out.\n");
 }    /*  END OF DATA STORE   */

void paraset()
 {
  int i;
  printf("\nSet parameter for simulation\n");
  for(i=1;i<=n;i++)
    {printf(" p[%d] = ",i),scanf("%lf",&p[i]);}
 }